# YASARA MOVIE
# TOPIC:    4. Molecular Modeling
# TITLE:    4.2. Knowledge-based potentials
# REQUIRES: Structure 20.5.1
# AUTHOR:   Elmar Krieger 
# LICENSE:  GPL
# DESCRIPTION: An overview of knowledge-based potentials. What they are, how to visualize them and how to use them.

RequireVersion 20.5.1
# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
OnError Continue
# Don't show console and head up display
Console Off
HUD Off
# Set default parameters
CoordSys Left
Projection perspective
StickRadius 50
BallStickRadius 50,50
AtomTexture 1
Fog 80
AtomSize 100
UserInput Off
ColorBG White
ColorBonds Order
PointPar Radius=7.00,Plastic=Yes
EnergyUnit kJ/mol
ColorPar Property,Min,Yellow,0.000
ColorPar Property,Max,Blue,1.000
LabelPar Font=Arial
LightSource Alpha=060,Gamma=039,Ambience=070,Shadow=040
if Structure and not Movie
  ForceField YASARA
# Load background
Clear
LoadPNG sky
ShowImage sky,0,0,1024,768

#Go EnergiesAndZScores
#Go DirectionPotentials
#Go DistancePotentials
#Go ShowingPotentials
#Go PhiPsiChi1Potential
#Go AboutPotentials

#
# The Intro
#
Intro:
LoadSce intro
ColorFog 606000
#go skip
ShowImage sky,0,100,800,600
kbpsize=400.
# Rotate around Yami's foot
CenterAtom 424
AutoRotateObj Yami,Y=1
# Zoom in on Yami while he jumps
for i=1 to 27
  for j=0 to 180 step 4
    MoveObj Yami,Y=(cos j)
    Wait 1
  if i==1
    AutoMove Y=-0.1
  if i==21  
    AutoMove Y=0
    AutoRotateObj Yami,Y=0
    # Get the current orientation of the knowledge-based potentials,
    # which must become Alpha=90/Beta=0/Gamma=0 during the next steps
    a,b,c=OriObj 2
    AutoRotate Y=(-b/92)
    AutoPos 172,-170,-235,Steps=92,Wait=No
    AutoMoveImage sky,112,100,800,600,Steps=92,Cycle=Off
  if i==23
    AutoRotate Y=0,Z=(-c/92)
  if i==25
    AutoRotate X=((90.-a)/92),Z=0
    AutoMoveImage sky,112,250,800,600,Steps=92,Cycle=Off
AutoRotate X=0
AutoPosObj Yami,X=-74.6,Y=49.1,Z=275.8,Steps=30,Wait=No
AutoOriObj Yami,0,-90,0,Steps=30
Wait 100
# Yami jumps down
CenterObj Yami
for i=0 to 180 step 4
  MoveObj Yami,X=1,Y=(cos i)
  RotateObj Yami,Z=-4
  Wait 1 
steps=100
AutoMoveImage sky,112,300,800,600,Steps=(steps),Cycle=Off
AutoPosObj Yami,X=-150,Y=-20,Z=320,Steps=(steps),Wait=No
AutoOriObj Yami,90,0,-90,Steps=(steps),Wait=No
x,y,z=PosObj 2
AutoMoveObj !Yami,((-121.374-x)/steps),((-138.230-y)/steps),((-789.874-z)/steps)
Wait (steps)
# Yami slides along the knowledge-based potentials
AutoMoveObj all,0,0,0
for i=1 to 3
  for j=0 to 359
    PosObj Yami,X=(cos (j+180)*125-25),Y=(sin (j*2+90)*40-60),Z=320
    OriObj Yami,Alpha=90,Beta=(-sin j*45),Gamma=(90-abs (180-j))
    if i==1 and j==140
      # Start moving the mountains
      AutoMove Z=(kbpsize/360)
      AutoMoveImage sky,0,250,1024,768,Steps=(120+360+120),Cycle=Off
    if j>=50 and j<85
      # Move up
      MoveObj Yami,Y=((sin (270.+(-50.+j)*180/35)+1)*40)
    if j>=85 and j<120
      # Little jump in between
      MoveObj Yami,Y=(sin ((-50.+j)*180/70)*80)
    if j>=120 and j<240
      MoveObj Yami,Y=((-1.+(abs (180.-j)/60))*100)
    if j==120 and i==3
      break 2
    Wait 1
AutoMove Z=0
AutoOriObj Yami,0,0,0,Steps=20,Wait=No
for i=0 to 22
  MoveObj Yami,Y=0.3,Z=(-cos (i*2)*3)
  Wait 1
# Create the title
titlelist='KNOWLEDGE','BASED','POTENTIALS'
SurfPar Resolution=4,Molecular=Gaussian,Radii=Solvation
for i=1 to count titlelist
  title(i)=BuildText Font=Arial,Height=14,Stroke=50%,Spacing=1.6,Element=Carbon,Bonds=Yes,Text=(titlelist(i))
  HideObj (title(i))
  ColorObj (title(i)),Yellow
  PosObj (title(i)),Y=-50
  ShowSurfObj (title(i)),Molecular,dynamic,OutCol=c0c000,OutAlpha=100    
# Let the title fly in
AutoOriObj Yami,Beta=180,Steps=40
PosObj KNOWLEDGE,X=-400,Y=145,Z=460
AutoPosObj KNOWLEDGE,X=-105,Steps=100
PosObj BASED,X=315,Y=86,Z=430
AutoPosObj BASED,X=-40,Steps=100
PosObj POTENTIALS,X=47.6,Y=210,Z=402
AutoPosObj POTENTIALS,Y=37.4,Steps=50
# Let the 'POTENTIALS' swing after landing
damp=0.15e0
ShowButton Continue
for i=1 to 5
  for j=0 to 359 step 4
    scale=-cos j*damp
    if j<=180
      for k=18 to 1 step -1
        MoveAtom LocalX>(-k*5) and LocalX<(k*5) and Obj POTENTIALS,Y=(scale*sin (k*5))
    else
      MoveObj POTENTIALS,Y=(-cos j*damp*10)
    Wait 1
    if ContinueButton
      break 2
  damp=damp*0.7
# Take snapshot here, Eliza!
Wait ContinueButton
# Fly away through the 'O' in 'POTENTIALS'
AutoMoveImage sky,112,424,800,600,Steps=1000,Cycle=Off
AutoPos X=67,Y=-114,Z=134,Steps=300
AutoMove Y=-0.2,Z=-1.2
Wait 700
AutoMove 0,0,0

#
# PART 1: About knowledge-based potentials
# 
AboutPotentials:
DelAll
DelImage !sky
HideMessage
Fog 0
ColorFog Black
LightSource Alpha=034,Gamma=042,Ambience=070,Shadow=040
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title1
ShowImage title1,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title1,500,375,24,18,Alpha=0,Steps=500
UserInput On
Wait 100
# Load the Ramachandran plot and potentials
LoadSce ramaplotser
LoadYOb tripeptide.yob.gz
ScaleObj Energy SmoothEnergy,Z=0.6
PosObj All,Y=0
SwitchObj all,off
plotori='Alpha=43,Beta=340,Gamma=0'
# Load the text
LoadPNG kbptext
scale=20.
MakeImageObj Text,kbptext,(700/scale),(1400/scale)
# Define the 3D movement of the text
textori='Alpha=66,Beta=0,Gamma=0'
OriObj Text,(textori)
linestepy=12e0/scale
linestepz=26.7e0/scale
start()=-10.500/scale,-562.500/scale,30.900/scale
start()=-10.500/scale,-562.500/scale,30.900/scale-11.283
PosObj Text,(start1),(start2),(start3)
line=0
# Scroll the text along, the numbers below are the lines per paragraph
lenlist()=12,6,7,5,6,4,20, 9,6,4,5,3,5,6,6,5,20
for i=1 to count lenlist
  steps=lenlist(i)
  line=line+steps
  steps=steps*20
  textpos='X=(start1),Y=(start2+linestepy*line),Z=(start3+linestepz*line)'
  if i<14
    GrabObj !Text and !tripeptide
  SwitchObj Text,on
  AutoPosOriObj Text,(textpos),(textori),Steps=(steps),Wait=No
  if i==2
    # Load the tripeptide
    LoadYOb tripeptide
    PosObj tripeptide,X=25,Y=4,Z=15
    AutoRotateObj tripeptide,Y=0.5
    AutoPosObj tripeptide,X=0,Steps=(steps)
  elif i==3
    # Show the crosses
    AutoPosObj tripeptide,Z=250,Steps=(steps)
    DelObj tripeptide
    OriObj !Text,0,0,0
    PosObj !Text,40,4,35
    SwitchObj Squares2x2 Labels,On
    AutoPosObj !Text,X=0,Steps=80
    Wait 50
    ScaleObj Serines,Z=0
    SwitchObj Serines,On
  elif i==4
    # Show preferred areas
    Wait (steps)
    SwitchObj HotSpots,On
  elif i==6
    # Show probability
    AutoOriObj !Text,(plotori),Steps=(steps)
    Wait 50
    SwitchObj Probability,On
  elif i==7
    # Load the second part of the text
    AutoPosObj Text,Z=500,Steps=200,Wait=Yes
    DelImage kbptext
    LoadPNG kbptext2
    PosObj Text,(start1),(start2),(start3)
    line=0
    continue
  elif i==9
    # Show energy
    Wait (steps)
    SwitchObj Probability,Off
    SwitchObj Energy,On
  elif i==11
    # Show smooth energy
    Wait (steps)
    SwitchObj Energy,Off
    SwitchObj SmoothEnergy,On
  elif i==12
    # Click HUD
    HUD Obj
  elif i==13
    # Show tripeptide again
    HUD Off
    UserInput Off
    AutoPosObj !Text,Y=100,Z=450,Steps=(steps)
    DelObj !Text
    SwapObj 1,Text
    LoadYOb tripeptide
    PosObj tripeptide,X=25,Y=4,Z=15
    AutoPosObj tripeptide,X=0,Steps=100
    # Draw arrows around the Phi/Psi dihedral angles
    # X/Y/Z-positions of the two arrow circle centers
    Wait 50
    radius=2
    pos1='-0.6,4.3,15'
    pos2='0.7,4.1,15'
    # Tilt angles for the two arrow circles around the Z-axis
    tilt=32,-40.5
    # Create the circles
    for j=1 to 2
      # Every circle consists of eight arrows
      for k=0 to 315 step 45
        ShowArrow Point,0,(sin k*radius),(-cos k*radius),Point,0,(sin (k+45)*radius),(-cos (k+45)*radius),
                  Radius=0.1,Color=Green
      JoinObj (j+3)-(Objects),(j+2)
      PosObj (j+2),(pos(j))
      RotateObj (j+2),Z=(tilt(j))
    Wait 100
    AutoMove Y=-0.12,Z=-0.25
    Wait 50
    AutoMove Y=0,Z=0
    Wait 100
    AutoMove Y=0.12,Z=0.25
    Wait 50
    AutoMove Y=0,Z=0
  elif i==15
    DelObj Arrow
    # Energy minimization
    if Movie or not Structure
      ShowMessage "The rest of this tutorial requires functions that are only available in YASARA Structure. Click 'Continue' to end..."
      Wait ContinueButton
      HideMessage
      Clear
      Stop
    UserInput Off
    AutoPosObj tripeptide,X=6.3,Y=4,Z=15,Steps=(steps)
    ForceField YASARA,SetPar=Yes
    Cell Auto,Extension=5
    SwitchObj SimCell,off
    ShowKBP 13,15,Name=PhiPsi,Size=21.0
    OriObj PhiPsiS,Alpha=35,Beta=339,Gamma=10 
    PosObj PhiPsiS,X=-50,Y=0,Z=50
    AutoPosObj PhiPsiS,X=-16,Steps=100
  elif i==16
    # Interactive pulling
    Boundary Wall
    LongRange None
    PullForce 10
    TempCtrl Anneal
    Temp 0
    Sim On
  elif i==17
    # Done
    AutoPosObj Text,Y=-300,Z=500,Steps=200,Wait=Yes
    break
  else
    Wait (steps)
  if i<14
    GrabObj tripeptide
  Wait ContinueButton  
AutoPosObj !Text,Y=400,Z=450,Steps=250

#
# PART 2: Phi/Psi/Chi1 potential
# 
PhiPsiChi1Potential:
DelAll
DelImage !sky
HideMessage
Fog 0
ColorFog Black
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title2
ShowImage title2,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title2,500,375,24,18,Alpha=0,Steps=500
UserInput On
# Load Ramachandran plot potentials, tripeptide
LoadSce ramaplotser
LoadYOb tripeptide
size=18.
x,y,z=PosObj 1 
obj=MakeTextObj Chi1Labels,Width=(size*1.6),Height=(size*1.6)
Font Name=Arial,Height=5%,Color=008000,Spacing=2,Depth=1,DepthCol=000000
PosText 9%,48%,Justify=Left
Print "-180"
PosText 80%,48%
Print "180"
PosText 45%,48%
Print "Chi1"
PrintCon
# Show the serine Phi/Psi/Chi1 potential
ShowKBP 13,15,Name=PhiPsiChi1,Size=(size)
# Move the tripeptide in
SwitchObj !tripeptide,off
PosObj all,Z=6
PosObj tripeptide,X=15
ShowMessage "In addition to the backbone dihedral angles Phi and Psi, amino acids have also side-chain dihedrals named Chi1 to ChiN."
AutoPosObj tripeptide,X=0,Steps=100
Wait 50
Wait ContinueButton
# Show Chi1 dihedral
ShowMessage "The one and only side-chain dihedral angle of serine is Chi1, which is shown now..."
AutoOriObj tripeptide,Alpha=50,Beta=0,Gamma=0,Steps=100
radius=2
first=Objects+1
for i=0 to 315 step 45
  ShowArrow Point,0,(sin i*radius),(-cos i*radius),Point,0,(sin (i+45)*radius),(-cos (i+45)*radius),
            Radius=0.1,Color=Green
JoinObj Arrow,(first)
PosObj Arrow,Y=1.2,Z=6.5
OriObj Arrow,0,90,90
# Rotate the serine side-chain around the Chi1 dihedral
ShowButton Continue
while 1
  for i=180 to -179 step -1
    Dihedral 13,15,19,22,Set=(i)
    a,b,g=Ori
    if a+b+g==0
      # Rotating the arrows only works until the user rotates the scene
      RotateObj Arrow,Y=0.5
    if ContinueButton
      break 2
    Wait 1
DelObj Arrow
# Show Ramachandran plot again 
ShowMessage "The Chi1 dihedral angle can be added as a third dimension to the 2-dimensional Phi/Psi potential..."
AutoPosObj tripeptide,X=8,Y=3,Z=19,Steps=100,Wait=No
AutoOriObj tripeptide,50,0,0,Steps=100
GrabObj !tripeptide
OriObj !tripeptide,0,0,0
RotateObj Chi1Labels,Y=-90
PosObj !tripeptide,X=-80,Y=0,Z=120
SwitchObj Squares2x2 Serines HotSpots Labels Chi1Labels,on
AutoPosObj !tripeptide,X=-23,Steps=100
AutoPosObj !tripeptide,X=-10,Y=0,Z=38
AutoRotateObj !tripeptide,Y=-0.5
Wait ContinueButton
ShowMessage "As expected for rotation around a bond between two sp3 carbons, the serines cluster around three different Chi1 dihedrals."
Wait ContinueButton
ShowMessage "These are at -60, +60 and +180 degrees, called 'gauche-', 'gauche+' and 'trans'."
Wait ContinueButton
ShowMessage "Due to the periodic wrap-around at -180/+180 degrees, the 'trans' conformation is split in two halves." 
Wait ContinueButton
ShowMessage "This wrap-around becomes more obvious when looking at the knowledge-based potential extracted from these data..."
SwitchObj PhiPsiChi1S,On
Wait ContinueButton
ShowMessage "Click Effects > Point & line appearance to customize the display of the 3D potential."
Wait ContinueButton
AutoRotateObj !tripeptide,Y=0
MarkAtom 13,15,19,22
ShowMessage "The Chi1 dihedral angle has been marked. Right-click on the N atom marked white, then Geometry > Dihedral to adjust it."
Wait ContinueButton
ShowMessage "Note that you can still rotate the 3D potential to follow the red arrow, even while the 'Set dihedral angle' window is open." 
Wait ContinueButton
MarkAtom C Res 1,N Res 3
ShowMessage "To change Phi and Psi, mark atoms C of the previous and N of the next residue, right-click on C and Geometry > Bond dihedral."
Wait ContinueButton
HideMessage
AutoPosObj all,Y=30,Z=850,Steps=250

#
# PART 3: Showing potentials
# 
ShowingPotentials:
DelAll
DelImage !sky
HideMessage
Fog 0
ColorFog Black
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title3
ShowImage title3,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title3,500,375,24,18,Alpha=0,Steps=500
UserInput On
LoadYOb tripeptide
SwapObj tripeptide,6
PosObj tripeptide,X=25,Y=-3,Z=15
AutoPosObj tripeptide,X=0,Steps=100
ShowMessage "To show potentials involving a certain dihedral, you first have to select the dihedral by marking the two atoms at the core."
Wait ContinueButton
AutoOriObj tripeptide,Alpha=45,Steps=30
ShowMessage "Mark the core of serine's Phi dihedral angle:Click on the serine nitrogen,then keep <Ctrl> pressed and click on the Calpha atom..."
Wait ContinueButton
MarkAtom 13,15
GrabObj tripeptide
ShowMessage "Now right-click on the atom marked white, then click Geometry > Knowledge-based potential. Keep name 'All' and click OK."
Wait ContinueButton
if Objects==1
  ShowKBP 13,15,Name=All,Size=21.0
ShowMessage "The YASARA force field contains five potentials that involve the Phi dihedral angle of serine. The first three are new..."
AutoOriObj !tripeptide,Alpha=55,Steps=200,Wait=No
AutoPosObj !tripeptide,Y=12,Z=143,Steps=200
Wait ContinueButton
AutoPosObj !tripeptide,Y=-47,Steps=100
HUD Obj
# Show and explain the five potentials
textlist()="First the simplest one, the 1D potential of the Phi dihedral angle alone. The potential names are listed in the HUD...",
           "The name 'PhiAS' stands for 'Phi dihedral involving alanine and serine', since it includes the C atom of the preceding alanine.",
           "Second the 2D potential 'PsiPhiAS', which captures the coupling of the Psi dihedral in alanine and the following Phi in serine.",
           "The Psi dihedral of alanine is displayed along the horizontal X-axis, and the Phi dihedral of serine along the vertical Y-axis.",
           "Third the 2D potential 'PhiChi1S', which describes the coupling of the Phi and Chi1 dihedrals in the central serine.",
           "The Phi dihedral is displayed along the horizontal X-axis, and the Chi1 dihedral along the vertical Y-axis.",
           "Fourth the 2D potential 'PhiPsiS', described in part 1. By now it is obvious which dihedrals maps to which side of the potential:",
           "The order of the dihedrals in the potential name simply matches the order of the axes: first along X, then along Y and finally Z.",
           "So it's not surprising that the final 3D potential 'PhiPsiChi1S' from part 2 shows Phi along the horizontal X-axis...",
           "...Psi along the vertical Y-axis and Chi1 along the Z-axis pointing into the screen." 
for i=1 to 5
  ShowMessage (textlist(i*2-1))
  GrabObj (i)
  x=PosObj (i)
  AutoPosObj (i),X=0,Y=5,Z=40,Steps=100,Wait=No
  AutoOriObj (i),Alpha=(50-(i!=1)*40),Beta=0,Gamma=0,Steps=100
  Wait ContinueButton
  ShowMessage (textlist(i*2))
  Wait ContinueButton
  AutoPosObj (i),X=(x),Y=-47,Z=143,Steps=100,Wait=No
  AutoOriObj (i),Alpha=55,Beta=0,Gamma=0,Steps=100
GrabAll
HideMessage
AutoPos 0,58,1280,Steps=250

#
# PART 4: Distance-dependent potentials
# 
DistancePotentials:
DelAll
DelImage !sky
HideMessage
Fog 0
ColorFog Black
HUD off
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title4
ShowImage title4,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title4,500,375,24,18,Alpha=0,Steps=500
UserInput On
LoadPDB 1crn
Style Ribbon,Stick
PosObj 1crn,X=40,Z=26
Wait 50
AutoPosObj 1crn,X=0,Steps=100
ShowMessage "In addition to the dihedral potentials shown in the previous sections, the YASARA2 force field contains intermolecular potentials."
Wait ContinueButton
ShowMessage "These can be extracted from the PDB using the very same 'Boltzmann approach'.Click Simulation>Force field>YASARA2, OK."
Clean
Wait ContinueButton
ForceField YASARA2,SetPar=Yes
ShowMessage "To show an intermolecular packing potential, the two interacting atoms have to be marked. Let's look at two methyl groups..."
AutoPosObj 1,X=0,Y=-12,Z=24,Steps=100,Wait=No
AutoOriObj 1,Alpha=56,Beta=85,Gamma=20
Wait ContinueButton
ShowMessage "One CH3 belongs to Ile 25, one to Thr 21. Right-click on the flashing atom, then Geometry > Knowledge-based potential > OK."
MarkAtom 358,304
Wait ContinueButton
ShowMessage "This potential shows the (again inverted) energy as a function of the distance. It converges to zero at large distance on the left."
if Objects==1
  ShowKBP 358,304,Name=All,Size=21.0
LabelDis 304,358,Format='DIS',Height=0.7,Color=Yellow
Wait ContinueButton
ShowMessage 'The red arrow marks the current distance, which is close to the optimum. Each tick-mark along the vertical axis is 1 (EnergyUnit).'
Wait ContinueButton
ShowMessage "Right-click on the flashing atom, then Geometry > Distance. Note how the interaction energy turns positive below 3.5 A."
MarkAtom 300,304
Wait ContinueButton
ShowMessage "At around 7 A, there is another unfavorable region, which distinguishes this knowledge-based- from a simple VdW potential:"
Distance 300,304,Set=1.548
Distance 353,358,set=7.14
AutoOriObj 1,Alpha=56,Beta=32,Gamma=20,Steps=100,Wait=No
for i=1.55 to 7.14 step 0.05
  Distance 353,358,set=(i)
  Wait 1
Wait ContinueButton
ShowMessage "Now the CH3-CH3 distance is just not large enough to pack something useful (=hydrophobic) in between, which is unfavorable."
Wait ContinueButton
ShowMessage "The orientation of the potentials is permanently adapted to let them face upwards, and they are updated during a simulation."
AutoPosObj 1,0,0,42,Steps=100,Wait=No
AutoOriObj 1,Alpha=-114,Beta=232,Gamma=-13,Steps=100,Wait=No
for i=7.00 to 1.55 step -0.05
  Distance 353,358,set=(i)
  Wait 1
# Add random potentials
AutoRotate X=1
atomlist()=ListAtom Element !H
for i=1 to 10
  atom1=onein atomlist
  atom2=onein atomlist
  if atom1!=atom2
    ShowKBP (atom1),(atom2),Name=Packing1D
  Wait 10
Wait 300
AutoRotate X=0
AutoPosObj 1,X=0,Y=0,Z=30,Steps=100,Wait=No
AutoOriObj 1,Alpha=-114,Beta=232,Gamma=-13,Steps=100
Cell Auto,Extension=10
Boundary Periodic
SwitchObj SimCell,off
TempCtrl SteepDes
Sim On
Wait SpeedMax<3000
Temp 298K
TempCtrl Rescale
Wait ContinueButton
ShowMessage "Note however that the packing potentials are currently only used to calculate energies, NOT to calculate forces during an MD."
Wait ContinueButton
Sim Off
HideMessage
AutoPos 0,58,1280,Steps=250

#
# PART 5: Direction-dependent potentials
# 
DirectionPotentials:
DelAll
DelImage !sky
HideMessage
Fog 0
ColorFog Black
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title5
ShowImage title5,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title5,500,375,24,18,Alpha=0,Steps=500
PointPar Radius=7
UserInput On
LoadPDB 1crn
Style Ribbon,BallStick
PosObj 1crn,X=-40,Z=26
Wait 50
AutoPosObj 1crn,X=0,Steps=100
AutoRotateObj 1crn,Y=-0.2
ShowMessage "If a heavy atom in a standard residue is bound to two or more heavy atoms, the YASARA2 force field contains a 3D potential."
Wait ContinueButton
ShowMessage "Looking at the CZ atom in arginine, the bound heavy atoms allow to define a unique 3D coordinate system shown now..."
MarkAtom CZ Res Arg 10
argpos='X=0,Y=2.3,Z=4.87'
argori='Alpha=58.59, Beta=309.86, Gamma=-5.96'
AutoRotateObj 1crn,Y=0
AutoOriObj 1,(argori)
Wait ContinueButton
ShowMessage "For each position in this coordinate system,we can extract the probability of finding a certain other atom,convert it to an energy..."
for i=1 to 46
  if i!=10
    HideRes (i)
  Wait 1
# Show coordinate system
start()=PosAtom CZ Res Arg 10,CoordSys=Global
nepos()=PosAtom NE Res Arg 10,CoordSys=Global
len=5
for i=1 to 3
  dir(i)=(nepos(i)-start(i))*len
ShowArrow Start=Point,(start1),(start2),(start3),End=Point,(start1+dir1),(start2+dir2),(start3),Radius=0.3,Heads=1,Color=Red
ShowArrow Start=Point,(start1),(start2),(start3),End=Point,(start1+dir2),(start2-dir1),(start3),Radius=0.3,Heads=1,Color=Green
ShowArrow Start=Point,(start1),(start2),(start3),End=Point,(start1),(start2),(start3-1.344*len),Radius=0.3,Heads=1,Color=Blue
AutoRotate Y=-0.2
Wait ContinueButton
ShowMessage "...and obtain a 3D 'direction-dependent knowledge-based potential'. Let's look at the interaction of Arg with carbonyl groups..." 
Wait ContinueButton
ShowAtom Res 2 or Backbone Res 3 46
BallStick
ShowSecStr Tube
AutoRotate Y=0
AutoMoveObj Arrow,Y=-0.2
AutoRotateObj Arrow,0.1,0.2,0.4
AutoPosObj 1,(argpos),Steps=100,Wait=No
AutoOriObj 1,(argori),Steps=100
AddHyd
Dihedral 19,23,25,26,set=-108
ShowHBoAtom Sidechain Res Arg 10,Extend=Yes
ForceField YASARA2,SetPar=Yes
DelObj Arrow
Wait ContinueButton
ShowMessage "The interacting atoms are marked, now right-click on the flashing atom and Geometry > Knowledge-based potential, OK."
DelObj Arrow
PosObj 1,(argpos)
OriObj 1,(argori)
CenterAtom 138
MarkAtom 138,22
Wait ContinueButton
ShowMessage "This is the potential of a carbonyl oxygen around arginine's guanidinium group. 'Yellow' is low-energy, 'blue' is high energy."
if Objects==1
  ShowKBP 138,22,Name=All,Size=21
DelObj Packing1D
Wait ContinueButton
ShowMessage "Note how the hydrogen-bonding oxygens in the neighborhood all fit into the yellow low-energy regions nicely."
Wait ContinueButton
HideMessage
AutoPos 0,58,1280,Steps=250

#
# PART 6: Energies and Z-Scores
# 
EnergiesAndZScores:
DelAll
DelImage !sky
Fog 0
ColorFog Black
HideMessage
ShowImage sky,112,424,800,600,Alpha=100
LoadPNG title6
ShowImage title6,-2048,-1000,5120,3840,Alpha=100
AutoMoveImage title6,500,375,24,18,Alpha=0,Steps=500
LoadPDB 1crn
LoadPDB 1acp,Model=1
NameObj 2,1acp
Style Ribbon,BallStick
PosObj 1crn,X=-60,Z=60
OriObj 1crn,Alpha=97,Beta=96,Gamma=6
PosObj 1acp,X=60,Z=60
OriObj 1acp,Alpha=-40,Beta=134,Gamma=-32
LabelObj all,Format='OBJNAME',Height=4.0,Color=Yellow
Wait 50
AutoPosObj 1crn,X=-16,Steps=100,Wait=No
AutoPosObj 1acp,X=16,Steps=100
UserInput On
textcomlist="In the final part of the tutorial, let's look at a major application of knowledge-based potentials...",'pass',
            "Knowledge-based energy calculations. Click Edit > Clean > All to prepare the two proteins for force field parameter assignment.",'Clean',
            "Then add a simulation cell: Click Simulation > Define simulation cell, set size automatically, extend 10 A at each side, OK.",'Cell Auto,Extension=10',
            "Now select the YASARA2 force field: Click Simulation > Force field, pick 'YASARA2' and click 'OK .. set its default parameters'.",'ForceField YASARA2,SetPar=Yes',
            "Click Analyze > Energy > Potential energy:Object,double-click 'All' on the right,calculate individual components,tick all boxes,OK.",'pass',
            "Press <Space> to show.'Dihedral1D..3D' are the knowledge-based energies,which are also part of the normal dihedral energy.",'pass',
            "'Packing1D' is the distance-dependent energy from part 4, 'Packing3D' is the direction-dependent energy from part 5.",'pass',
            "If you compare the packing energies between 1CRN and 1ACP, you see that it's not really easy to tell which one is better.",'pass',
            "The reason is that energies depend strongly on the protein sequence, size and shape, and 1ACP is 70% larger than 1CRN.",'pass',
            "The solution are normalized energies expressed as Z-scores, i.e. the number of standard deviations away from the average.",'pass',
            "Click Analyze > Check > Object, double-click 'All', double-click 'Packing3D' to calculate the 3D packing Z-score.",'CheckObj all,Packing3D',
            "Now things are more obvious: the high resolution X-ray structure 1CRN is slightly better than the gold standard...",'pass',
            "...while the low resolution NMR structure 1ACP is almost eight standard deviations below the gold standard.",'pass',
            "You can also check individual residues: Click Analyze > Check > Residue, double-click 'All', double-click 'Packing3D'.",'CheckRes all,Packing3D', 
            "The check results are stored in the atomic property field. Visualize them by clicking View > Color > All, double-click 'Property'.",'Color Property',
            "As always, the color gradient runs from blue (normal) to yellow (hot spots to look at). Obviously, 1ACP needs more attention.",'pass'
for i=1 to count textcomlist step 2
  ShowMessage (textcomlist(i))
  Wait ContinueButton
  (textcomlist(i+1))
  
AutoPos 0,58,1280,Steps=250
LoadSce yamimatrass
UserInput Off
Move Y=-44,Z=25
AutoRotate Y=0.5
AutoMove Z=0.25
ShowMessage "Click the <ContinueButton> to end this tutorial..."
Wait ContinueButton
Clear
  
"""
Additional data:

1) How to create the PhiPsi plot with WHAT IF for SER:

%SEQUEN:
Residues at position: 1 ALL
Residues at position: 2 ALL
Residues at position: 3 SER
Residues at position: 4 ALL
Residues at position: 5 ALL
Number of allowed mismatches= ( 0 )

%GRACHI
Please give the group number 1
Which hits should be used ( 1- 6399 ) 1 1000
Give the position in the fragment 3
Give three angles out of <FPO12345> FP1
Give the number of the MOL-object ( 1 ) 1
Give the name of the MOL-item : (GRACHI)
Give the number of the MOL-object ( 2 )
Give the name of the MOL-item : (GRACHI1)

You can plot the allowed boundaries in the Phi-Psi plane
Give the number of the MOL-object ( 3 )
Give the name of the MOL-item : (GRACHI2)


2) Creating the labels for the Ramachandran plot:

# Label the Ramachandran plot
len=18.
x,y,z=PosObj 1 
#ShowArrow Start=Point,X=(-len/2),Y=(-len/2-2),Z=(z),
#            End=Point,X= (len/2),Y=(-len/2-2),Z=(z),Radius=0.5,Heads=1,Color=Yellow
#ShowArrow Start=Point,X=(-len/2-2),Y=(-len/2),Z=(z),
#            End=Point,X=(-len/2-2),Y=(len/2), Z=(z),Radius=0.5,Heads=1,Color=Yellow
obj=MakeTextObj Labels,Width=(len*1.6),Height=(len*1.6)
PosObj (obj),Z=(z)
Font Name=Arial,Height=5%,Color=008000,Spacing=2,Depth=1,DepthCol=000000
PosText 4%,85%,Justify=Left
Print " 180"
PosText 4%,10%
Print "-180"
PosText 85%,10%
Print "180"
PosText 4%,48%
Print " Psi"
PosText 45%,10%
Print "Phi"
PrintCon

LoadYob tripeptide
SwapRes 2 Obj 10,Ala
ForceField YASARA
for i=1 to 8
  obj = ShowKBP N Res Ala 2 Obj 10,CA Res Ala 2 Obj 10,Name=PhiPsi,Size=400.0
  TransferObj (obj),(i+1)
  SwapObj (obj),(i+1)
  DelObj (obj)
DelObj tripeptide  

3) The 3D text, with line feeds, 35pt, 1024x2048 pixels, Font Sans: 

Knowledge Based Potentials
  
The parameters of molecular dynamics force fields like AMBER
have been fitted to reproduce results from quantum mechanical
calculations. In the YAMBER force fields, these parameters
have been further optimized to yield energy minima close to native
protein structures. In YASARA Structure, two new force fields take
this approach one step further: The parameters are extracted
directly from today's highest resolution protein structures in the
PDB - from the golden core of our knowledge about proteins.

Extracting a force field from known protein structures requires five
steps. These will be summarized now, drawing on the example of
a serine residue and the dihedral angle components of the force
field. One serine is shown above as part of an Ala-Ser-Ala
tripeptide.

Step 1 - Data collection: We loop over our gold standard of high
resolution protein structures, looking for serine residues. For
every hit, we calculate the dihedral angles and store them. 
Looking only at the two backbone dihedrals Phi and Psi, this gives
us the plot above, where every cross represents one serine
residue in our database.

Not surprisingly, this famous 'Ramachandran plot' has preferred
areas, marked now above. The serines in beta sheets populate
the red regions, while serines in alpha helices cluster in the blue
region. Coil and loop serines can be anywhere.

Step 2 - Probability calculation: If we subdivide the plot above into
squares (2x2 are shown above, but 64x64 are used in YASARA),
count the number of crosses per square and divide by the total
number of crosses, we get the probability of finding a serine in
each square.

This probability can easily be visualized, yellow tops correspond
of course to the most common regions in the Ramachandran plot,
the hot spots.

====================================================================

Step 3 - Energy calculation:Fortunately, the probability distribution
above can be converted to an energy quite easily. Boltzmann's
formula states that the probability is proportional to exp(-E/kT),
where E is the energy, T the temperature and k the Boltzmann
constant ('exp' is the exponential function). So we only need to
turn this formula around to obtain the energy:
                           E ~ -log(probability)*kT.

The corresponding energy is shown above. To aid visualization,
YASARA drops the minus sign from the previous formula.
So probability and energy point in the same direction, while they
should normally be opposed (high probability = low energy).
Note how the logarithm emphasizes the less populated regions.    

Step 4 - Smoothing: The energy landscape above is still quite
rough, and would thus add a lot of noise to energy and especially
force calculations.

It is therefore replaced by a smooth function that approximates
the overall shape.What we obtain is the final knowledge-based
potential, a potential energy function based on knowledge
extracted from known protein structures.

<LeftClick> in the 'Vis' column of the scene content HUD to have
a final look at the various plots.

Step 5 - Force calculation: Thanks to the smoothing procedure,
we can now calculate a meaningful first derivative at each
position in the potential, which is the force acting on the Phi/Psi
dihedral angles of our serine residue.
  
Let's use this knowledge-based force to energy minimize the
tripeptide. Click Simulation > Force field > YASARA, OK. The
YASARA force field has been derived from the YAMBER3 force
field by adding the knowledge-based dihedral forces explained
here.
  
The red arrow indicates the current Phi/Psi combination of the
middle serine residue. Click Options > Experiment > Minimization.
Note how the arrow moves towards the yellow low energy regions
in the Ramachandran plot, sometimes jumping around when it
crosses a periodic boundary of dihedral angle space.

Click on any atom, then hold down <Ctrl> and the right mouse
button to pull it. Note that electrostatic and VdW interactions
contribute significantly, so the red arrow does not simply move
to the closest energy minimum of the dihedral potential.

"""  
  
    
